@@ -1,3 +1,5 @@
+;;; helpful-unit-test.el -*- lexical-binding: t; -*-
+
 (require 'ert)
 (require 'edebug)
 (require 'helpful)
@@ -133,6 +135,18 @@ bar")))
     (helpful-function #'test-foo-defined-interactively)
     (should (equal (buffer-name) "*helpful function: test-foo-defined-interactively*"))))
 
+(ert-deftest helpful--variable-defined-at-point ()
+  ;; The happy case with valid code.
+  (with-temp-buffer
+    (insert "(defvar foo nil)")
+    (goto-char (1+ (point-min)))
+    (should (eq (helpful--variable-defined-at-point) 'foo)))
+  ;; Ensure we don't crash if the source code isn't valid.
+  (with-temp-buffer
+    (insert "(defvar foo nil")
+    (goto-char (1+ (point-min)))
+    (helpful--variable-defined-at-point)))
+
 (ert-deftest helpful--edebug-fn ()
   "We should not crash on a function with edebug enabled."
   (let ((edebug-all-forms t)
@@ -405,7 +419,7 @@ variables defined without `defvar'."
   ;; Emacs instance.
   (skip-unless (null (get-buffer "python.el.gz")))
 
-  (-let [(buf pos opened) (helpful--definition 'python-indent-offset nil)]
+  (-let [(buf _pos opened) (helpful--definition 'python-indent-offset nil)]
     (should (bufferp buf))
     (should opened)))
 
@@ -423,7 +437,7 @@ variables defined without `defvar'."
 		   (lambda (_format-string &rest _args))))
           (eval (eval-sexp-add-defvars (edebug-read-top-level-form)) t))
 
-        (-let [(buf pos opened) (helpful--definition 'test-foo-edebug-defn t)]
+        (-let [(buf _pos _opened) (helpful--definition 'test-foo-edebug-defn t)]
           (should buf))))))
 
 (ert-deftest helpful--definition-defstruct ()
@@ -554,6 +568,13 @@ associated a lambda with a keybinding."
     (with-current-buffer buf
       (helpful-update))))
 
+(ert-deftest helpful--unnamed-func-with-docstring ()
+  (let* ((fun (lambda (x) "Foo" x))
+         (buf (helpful--buffer fun t)))
+    ;; Don't crash when we show the buffer.
+    (with-current-buffer buf
+      (helpful-update))))
+
 (ert-deftest helpful--unnamed-compiled-func ()
   "Ensure we handle unnamed byte-compiled functions.
 
@@ -650,9 +671,9 @@ associated a lambda with a keybinding."
 (ert-deftest helpful--keymap-keys--anonymous-fns ()
   (let* ((keymap (make-keymap)))
     (define-key keymap (kbd "a")
-      (lambda () (message)))
+      (lambda () (interactive) (message "")))
     (define-key keymap (kbd "a")
-      (byte-compile-sexp (lambda () (message))))
+      (byte-compile (lambda () (interactive) (message ""))))
 
     ;; Don't crash on anonymous functions in a keymap.
     (helpful--keymap-keys keymap)))
@@ -745,14 +766,14 @@ in."
       (c . (11))))))
 
 (ert-deftest helpful--source ()
-  (-let* (((buf pos opened) (helpful--definition #'helpful--source t))
+  (-let* (((buf pos _opened) (helpful--definition #'helpful--source t))
           (source (helpful--source #'helpful--source t buf pos)))
     (should
      (s-starts-with-p "(defun " source))))
 
 (ert-deftest helpful--source-autoloaded ()
   "We should include the autoload cookie."
-  (-let* (((buf pos opened) (helpful--definition #'helpful-at-point t))
+  (-let* (((buf pos _opened) (helpful--definition #'helpful-at-point t))
           (source (helpful--source #'helpful-at-point t buf pos)))
     (should
      (s-starts-with-p ";;;###autoload" source))))
@@ -761,7 +782,7 @@ in."
   "We should return the raw sexp for functions where we can't
 find the source code."
   (eval '(defun test-foo-defined-interactively () 42))
-  (-let* (((buf pos opened) (helpful--definition #'test-foo-defined-interactively t)))
+  (-let* (((buf pos _opened) (helpful--definition #'test-foo-defined-interactively t)))
     (should
      (not
       (null
@@ -1050,3 +1071,17 @@ find the source code."
   (should (s-contains-p "Implementations" (buffer-string)))
   (should (s-contains-p "((l xref-file-location))" (buffer-string)))
   (should (s-contains-p "((l xref-buffer-location))" (buffer-string))))
+
+(defun helpful--boring-advice (orig-fn &rest args)
+  (apply orig-fn args))
+
+(advice-add 'ruby-mode :around #'helpful--boring-advice)
+
+(ert-deftest helpful--autoload-functions-with-advice ()
+  "Ensure that we can describe an autoloaded function
+that has advice attached before it is loadedl."
+  (helpful-function 'ruby-mode))
+
+(ert-deftest helpful--tree-any-p ()
+  (should (helpful--tree-any-p (lambda (x) (eq x 1)) '((((1))))))
+  (should (helpful--tree-any-p (lambda (x) (eq x 1)) (cons 2 1))))
